<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author  Tim Gatzky <info@tim-gatzky.de>
 * @package  pct_customelements
 * @link  http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Core;

/**
 * Imports
 */
use PCT\CustomElements\Core\AttributeFactory as AttributeFactory;
use PCT\CustomElements\Helper\ControllerHelper as ControllerHelper;

/**
 * Class file
 * Vault
 * Provide methods to load and save attribute information to the database
 */
class Vault
{
	/**
	 * Vault table
	 * @var string
	 */
	protected static $strTable   = 'tl_pct_customelement_vault';

	/**
	 * Binary attribute types
	 * @var array
	 */
	protected static $arrBinary  = array('file','files');

	/**
	 * Current object instance (Singleton)
	 * @var object
	 */
	protected static $objInstance;

	/**
	 * Instantiate this class and return it (Factory)
	 * @return object
	 * @throws Exception
	 */
	public static function getInstance()
	{
		if (!is_object(static::$objInstance))
		{
			static::$objInstance = new static();
		}

		return static::$objInstance;
	}

	/**
	 * Save the wizzard data
	 * @param integer
	 * @param string
	 * @return mixed
	 */
	public static function saveWizardData($arrValue=array(),$intPid,$strTable='tl_content',$arrOptions=array())
	{
		$objDatabase = \Contao\Database::getInstance();

		$time = time();

		$arrSet = array
		(
			'pid'  => $intPid,
			'tstamp' => $time,
			'type'  => 'wizard',
			'data_blob' => serialize($arrValue),
		);
		
		if(count($arrOptions) > 0)
		{
			$arrSet = array_merge($arrSet,$arrOptions);
		}
		
		$strType = 'wizard';
		if(strlen(trim($arrOptions['type'])) > 0)
		{
			$strType = trim($arrOptions['type']);
			$arrSet['type'] = $strType;
		}
		else
		{
			$arrSet['type'] = $strType;
		}
		
		$intGenericAttribute = $arrOptions['attr_id'] ?: 0;
			
		// check if entry exists
		$objEntry = $objDatabase->prepare("SELECT id FROM ".static::$strTable." WHERE pid=? AND source=? AND type=? AND (attr_id=? OR attr_id=0)")
		->limit(1)
		->execute($intPid,$strTable,$strType,$intGenericAttribute);

		if($objEntry->numRows < 1)
		{
			$arrSet['source'] = $strTable;
			
			if(!is_array($arrSet['data']) || empty($arrSet['data']))
			{
				$arrSet['data'] = '';
			}
			
			// insert new vault entry
			$objStatement = $objDatabase->prepare("INSERT INTO ".static::$strTable." %s")->set($arrSet);
		}
		else
		{
			// update entry
			$objStatement = $objDatabase->prepare("UPDATE ".static::$strTable." %s WHERE id=".$objEntry->id)->set($arrSet);
		}

		$objStatement->execute();
		
		if(!\Contao\Environment::get('isAjaxRequest'))
		{
			// log
			ControllerHelper::callstatic('log',array('A new customelement widget has been saved for: '.$strTable.'.id='.$intPid,__METHOD__, TL_GENERAL));
		}
	}
	
	
	/**
	 * Return the wizard data as array
	 * @param integer	ID of the element
	 * @param string	Name of the source / table
	 * @param integer	ID of the generic attribute
	 */
	public static function getWizardData($intPid,$strTable,$intGenericAttribute=0)
	{
		if($intPid < 1 || strlen($strTable) < 1)
		{
			return array();
		}
		
		if(Cache::getWizard($intPid,$strTable,$intGenericAttribute))
		{
			return Cache::getWizard($intPid,$strTable,$intGenericAttribute);
		}
		
		$objWizard = \Contao\Database::getInstance()->prepare("SELECT data_blob FROM tl_pct_customelement_vault WHERE pid=? AND source=? AND type=? ".($intGenericAttribute ? " AND attr_id=? " : "") )->limit(1)->execute($intPid,$strTable,'wizard',$intGenericAttribute);
		if($objWizard->numRows < 1)
		{
			return array();
		}
		
		// add to the cache
		Cache::addWizard($intPid,$strTable,$intGenericAttribute,$objWizard->data_blob);
		
		return \deserialize($objWizard->data_blob);
	}

	/**
	 * Return the value of an attribute
	 * @param integer
	 * @param string
	 * @return mixed
	 */
	public static function getAttributeValueByUuid($strField,$intPid,$strTable,$arrOptions=array())
	{
		$arrWizard = static::getWizardData($intPid,$strTable,$arrOptions['genericAttribute']);
		if(empty($arrWizard))
		{
			return null;
		}
		
		// handle generic child option fields
		$strChildField = '';
		if(strlen(strpos($strField, '_')))
		{
			$arrFieldName = explode('_', $strField);
			$strField = $arrFieldName[0];
			$strChildField =  $arrFieldName[1];
		}
		
		//-- look up from wizard
		if(isset($arrWizard['values'][$strField]) && strlen($strChildField) < 1)
		{
			// binary data
			if($arrOptions['saveDataAs'] == 'binary')
			{
				// is up to date
				if(\Contao\Validator::isBinaryUuid($arrWizard['values'][$strField]))
				{
					return $arrWizard['values'][$strField];
				}
				else if(\Contao\Validator::isStringUuid($arrWizard['values'][$strField]))
				{
					if(class_exists('\Contao\StringUtil'))
					{
						return \Contao\StringUtil::uuidToBin($arrWizard['values'][$strField]);
					}
					else
					{
						return \Contao\StringUtil::uuidToBin($arrWizard['values'][$strField]);
					}
				}
			}
			else
			{
				return $arrWizard['values'][$strField];
			}
		}
		
		if(isset($arrWizard['values'][$strField.'_'.$strChildField]) && strlen($strChildField) > 0)
		{
			return $arrWizard['values'][$strField.'_'.$strChildField];
		}
		
		//--
		$strDataField = 'data';
		// handle dynamic data, child options always use the data column
		if(strlen($arrOptions['saveDataAs']) > 0 && strlen($strChildField) < 1)
		{
			switch($arrOptions['saveDataAs'])
			{
				case 'binary':
					$strDataField = 'data_binary';
					break;
				case 'blob':
					$strDataField = 'data_blob';
					break;
				case 'text':
					$strDataField = 'data_text';
					break;
				default: break;
			}
		}
		
		$objValue = \Contao\Database::getInstance()->prepare("SELECT id,".$strDataField." FROM ".static::$strTable." WHERE pid=? AND source=? AND attr_uuid=?". ($strChildField ? " AND cattr_uuid='".$strChildField."'" : " AND cattr_uuid=''") )->limit(1)->execute($intPid,$strTable,$strField);
		if($objValue->numRows < 1)
		{
			switch($strDataField)
			{
				case 'data':
				case 'text':
				default:
					return '';
					break;
				case 'blob':
				case 'binary':
					return null;
					break;
			}
			return null;
		}
		
		// update binary values in the wizard data
		if(isset($arrWizard['values'][$strField]) && $arrOptions['saveDataAs'] == 'binary')
		{
			$value = $objValue->{$strDataField};
			if(\Contao\Validator::isBinaryUuid($value))
			{
				if(class_exists('\Contao\StringUtil'))
				{
					$value =  \Contao\StringUtil::binToUuid($value);
				}
				else
				{
					$value = \Contao\StringUtil::binToUuid($value);
				}
				
				$arrWizard['values'][$strField] = $value;
				static::saveWizardData($arrWizard,$intPid,$strTable);
			}
		}
				
		return $objValue->{$strDataField};
	}


	/**
	 * Store the value of an attribute in the vault
	 * @param mixed  Value
	 * @param integer Id of the element saving the attribute
	 * @param string Unique id / name of the attribute
	 * @param string Source table of the parent element saving the attribute
	 * @param string Set the storage column
	 */
	public static function setAttributeValue($varValue,$intPid,$strField,$strTable='tl_content',$arrOptions=array())
	{
		if(strlen($strField) < 1)
		{
			return;
		}
		
		$objDatabase = \Contao\Database::getInstance();
		$objStatement = null;

		$time = time();

		// handle generic child option fields
		$strChildField = '';
		if(strlen(strpos($strField, '_')))
		{
			$arrFieldName = explode('_', $strField);
			$strField = trim($arrFieldName[0]);
			$strChildField =  trim($arrFieldName[1]);
		}

		$arrSet = array
		(
			'pid'   		=> $intPid,
			'tstamp'  		=> $time,
			'type'   		=> $arrOptions['type'],
			'attr_uuid'  	=> $strField,
			'cattr_uuid'  	=> $strChildField ? $strChildField : '',
			'attr_id'  		=> $arrOptions['attr_id'],
		);

		switch($arrOptions['saveDataAs'])
		{
			case 'binary':
				if (version_compare(VERSION, '3.2', '<'))
				{
					$arrSet['data_binary'] = $varValue;
					break;
				}
				
				if (\Contao\Validator::isStringUuid($varValue))
				{
					if(class_exists('\Contao\StringUtil')) 
					{ 
						$varValue = \Contao\StringUtil::uuidToBin($varValue); 
					}
					else 
					{ 
						$varValue = \Contao\StringUtil::uuidToBin($varValue); 
					}
				}
				
				$arrSet['data_binary'] = $varValue;
				break;
			case 'blob':
				$arrSet['data_blob'] = $varValue;
				break;
			case 'text':
				// value should not be NULL
				if($varValue === NULL) {$varValue = '';}
				$arrSet['data_text'] = $varValue;
				break;
			default:
				// value should not be NULL
				if($varValue === NULL) {$varValue = '';}
				$arrSet['data'] = $varValue;
				break;
		}
		
		if(!$arrOptions['bypassCallback'])
		{
			// HOOK allow manipulations on the set array
			$arrSet = \PCT\CustomElements\Core\Hooks::callstatic('storeValueHook',array($arrOptions['attr_id'],$arrSet));
		}
		
		// check if entry exists
		$objEntry = $objDatabase->prepare("SELECT id FROM ".static::$strTable." WHERE pid=? AND attr_uuid=? AND source=?" . ($strChildField ? " AND cattr_uuid='".$strChildField."'" : " AND cattr_uuid=''") )
			->limit(1)
			->execute($intPid,$strField,$strTable);

		if($objEntry->numRows < 1)
		{
			$arrSet['source'] = $strTable;

			// insert new vault entry
			$objStatement = $objDatabase->prepare("INSERT INTO ".static::$strTable." %s")->set($arrSet);
			$objStatement->execute();
		}
		else
		{
			// update entry
			$objStatement = $objDatabase->prepare("UPDATE ".static::$strTable." %s WHERE id=".$objEntry->id)->set($arrSet);
			$objStatement->execute();
		}
	}
	
	
	/**
	 * Return boolean true if a field has been saved before
	 * @param string
	 * @param integer
	 * @param string
	 */
	public static function fieldExists($strField,$intPid,$strTable)
	{
		$arrWizard = static::getWizardData($intPid,$strTable);
		
		// widget has not been saved before at all
		if(!is_array($arrWizard['uuids']))
		{
			return false;
		}
		
		if(in_array($strField, $arrWizard['uuids']))
		{
			return true;
		}
		
		// fallback
		$objEntry = \Contao\Database::getInstance()->prepare("SELECT id FROM tl_pct_customelement_vault WHERE attr_uuid=? AND pid=? AND source=?")->limit(1)->execute($strField,$intPid,$strTable);
		if($objEntry->numRows > 0 || $objEntry->id > 0)
		{
			return true;
		}
		
		return false;
	}
	

	/**
	 * Public static function to retrieve a value from the vault
	 * @param string
	 * @param integer
	 * @param string
	 */
	public static function getValue($strField,$intPid,$strTable,$arrOptions=array())
	{
		return static::getInstance()->getAttributeValueByUuid($strField,$intPid,$strTable,$arrOptions);
	}

	/**
	 * Remove attributes from vault
	 * @param array
	 */
	public static function removeAttributes($arrUuids,$intPid=0,$bolLog=true)
	{
		if(empty($arrUuids))
		{
			return;
		}

		if(!is_array($arrUuids))
		{
			$arrUuids = array($arrUuids);
		}

		\Contao\Database::getInstance()->execute("DELETE FROM ".static::$strTable." WHERE ".($intPid > 0 ? " pid=".$intPid." AND ":"").\Contao\Database::getInstance()->findInSet('attr_uuid',$arrUuids));
	
		// log
		if($bolLog)
		{
			ControllerHelper::callstatic('log',array('DELETE FROM '.static::$strTable.' WHERE attr_uuid IN('.implode(',', $arrUuids).')',__METHOD__, TL_GENERAL));
		}
	}

	/**
	 * Remove attributes and generic attributes from vault
	 * @param array
	 */
	public static function removeAttributesById($arrIds,$bolLog=true)
	{
		if(empty($arrIds))
		{
			return;
		}

		if(!is_array($arrIds))
		{
			$arrIds = array($arrIds);
		}

		\Contao\Database::getInstance()->execute("DELETE FROM ".static::$strTable." WHERE ".\Contao\Database::getInstance()->findInSet('attr_id',$arrIds));
	
		// log
		if($bolLog)
		{
			ControllerHelper::callstatic('log',array('DELETE FROM '.static::$strTable.' WHERE attr_id IN('.implode(',', $arrIds).')',__METHOD__, TL_GENERAL));
		}
	}


	/**
	 * Remove attributes and generic attributes from vault
	 * @param array
	 */
	public static function removeById($arrIds,$bolLog=true)
	{
		if(empty($arrIds))
		{
			return;
		}

		if(!is_array($arrIds))
		{
			$arrIds = array($arrIds);
		}

		\Contao\Database::getInstance()->execute("DELETE FROM ".static::$strTable." WHERE ".\Contao\Database::getInstance()->findInSet('id',$arrIds));
	
		// log
		if($bolLog)
		{
			ControllerHelper::callstatic('log',array('DELETE FROM '.static::$strTable.' WHERE id IN('.implode(',', $arrIds).')',__METHOD__, TL_GENERAL));
		}
	}


	/**
	 * Clean the vault from obsolete data
	 * @param string
	 * @param array
	 * @param string
	 * @param array
	 * called from daily cron job HOOK
	 */
	public function purgeVault()
	{
		// log: start
		#ControllerHelper::callstatic('log',array('PURGING Vault...','', TL_GREEN));
		
		$objCustomElement = \PCT\CustomElements\Core\CustomElements::getInstance();
		
		// get all allowed tables
		$arrTables = $objCustomElement->getAllowedTables();
		if(empty($arrTables))
		{
			// log finished
			ControllerHelper::callstatic('log',array('PURGING Vault... Finished: Nothing to do.',__METHOD__, TL_CRON));
			return;
		}
		
		$arrObsolete = array(); // collect vault ids that can be removed
		$objDatabase = \Contao\Database::getInstance();
		foreach($arrTables as $strSource)
		{
			// fetch vault data
			$objVault = $objDatabase->prepare("SELECT * FROM ".static::$strTable." WHERE source=?")->execute($strSource);
			if($objVault->numRows < 1)
			{
				continue;
			}
			
			$selectfield = $objCustomElement::getSelectionField($strSource);
			$arrCustomElements = $objCustomElement->getCustomElements($strSource);
			
			// check if the corresponding parent record still exists
			while($objVault->next())
			{
				$objParent = $objDatabase->prepare("SELECT id,".$selectfield." FROM ".$strSource." WHERE id=?")->execute($objVault->pid);
				if($objParent->numRows < 1)
				{
					$arrObsolete[] = $objVault->id;
				}
				
				// check if the element still is a customelement
				$strCustomElement = $objParent->$selectfield;
				if(!in_array($strCustomElement, $arrCustomElements))
				{
					$arrObsolete[] = $objVault->id;
				}
			}
		}
		
		$arrObsolete = array_unique($arrObsolete);
		
		if(count($arrObsolete) < 1)
		{
			// log finished
			ControllerHelper::callstatic('log',array('PURGING Vault... Finished: Nothing to do.',__METHOD__, TL_CRON));
			return;
		}
		
		static::removeById($arrObsolete,false);
		
		// log finished
		ControllerHelper::callstatic('log',array('PURGING Vault... Finished: Removed:'.implode(',', $arrObsolete),__METHOD__, TL_CRON));
	}
}