<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_iconpicker
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\IconPicker;

/**
 * Class file
 * TablePage
 */
class TablePage extends \Contao\Backend
{
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('BackendUser', 'User');
		
		// add the stylesheets to the page
		$objPicker = new IconPicker();
		$objPicker->addStyleSheetsToPage();
	}

	/**
	 * Save font icon class in real page class field
	 */
	public function setCssClassOnSave($varValue, \DataContainer $objDC)
	{
		$varValue = $this->getFormattedClassFieldValue($varValue, $objDC);
		
		return $varValue;
	}
	
	/**
	 * Save font icon class in real page class field
	 */
	public function setCssClassOnLoad($varValue, \DataContainer $objDC)
	{
		$varValue = $this->getFormattedClassFieldValue($varValue, $objDC);
		
		// manually update the database here
		\Contao\Database::getInstance()->prepare("UPDATE ".$objDC->table." %s WHERE id=?")->set(array('cssClass'=>$varValue))->execute($objDC->id);
		
		return $varValue;
	}

	/**
	 * Prepare the class field array and return it
	 * @param string
	 * @param object
	 * @return array
	 */
	protected function getFormattedClassFieldValue($varValue, \DataContainer $objDC)
	{
		$objActiveRecord = \Contao\Database::getInstance()->prepare("SELECT * FROM tl_page WHERE id=?")->limit(1)->execute($objDC->id);

		// allow to enter a icon class manually
		if(!$objActiveRecord->addFontIcon)
		{
			return $varValue;
		}
		
		// get the selectors
		$arrSelectors = $GLOBALS['PCT_ICONPICKER']['iconClasses'] ?: array('.icon');
		if(!in_array('.icon', $arrSelectors))
		{
		   $arrSelectors[] = '.icon';
		}
		
		// custom icon selectors
		if(\Contao\Config::getInstance()->get('customIconClasses'))
		{
		   $arrSelectors = array_merge($arrSelectors, explode(',',\deserialize(\Contao\Config::getInstance()->get('customIconClasses'))) );
		}

		$arrClass = explode(' ', $varValue);
		
		foreach($arrClass as $i => $strClass)
		{
			$tmp = explode(' ', $objActiveRecord->fontIcon);
			if(in_array($strClass, $tmp))
			{
				unset($arrClass[$i]);
			}
			
			foreach($arrSelectors as $s)
			{
				$cls = '.'.$strClass;
				if(strlen(strpos($cls, $s)) > 0 || $s == '.'.$strClass.'-' || $s == $cls)
				{
					unset($arrClass[$i]);
				}
			}
		}
		
		// add the fonticon class
		if(strlen($objActiveRecord->fontIcon) > 0)
		{
			$arrClass[] = $objActiveRecord->fontIcon;
		}
		
		$arrClass = array_filter(array_unique($arrClass),'strlen');
		$varValue = implode(' ', $arrClass);
		
		return $varValue;
	}
	
}