<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright	Tim Gatzky 2013
 * @author  	Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 * @license		http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Backend;

/**
 * Imports
 */
use \Input;
use \Image;

/**
 * Class file
 * TableRevolutionSlider
 * Provide methods for table tl_revolutionslider
 */
class TableRevolutionSlider extends \Backend
{
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('BackendUser','User');
	}

	/**
	 * Check permissions
	 */
	public function checkPermission()
	{
		if ($this->User->isAdmin)
		{
			return;
		}
		
		// Set root IDs
		if (!is_array($this->User->revolutionsliders) || empty($this->User->revolutionsliders))
		{
			$root = array(0);
		}
		else
		{
			$root = $this->User->revolutionsliders;
		}
		
		$GLOBALS['TL_DCA']['tl_revolutionslider']['list']['sorting']['root'] = $root;
		
		// Check permissions to add sliders
		if (!$this->User->hasAccess('create', 'revolutionslidersp'))
		{
			$GLOBALS['TL_DCA']['tl_revolutionslider']['config']['closed'] = true;
		}
		
		// show all sliders if user has richts to create and no specific slider is selected
		if(in_array('create', $this->User->revolutionslidersp) && empty($this->User->revolutionsliders))
		{
			unset($GLOBALS['TL_DCA']['tl_revolutionslider']['list']['sorting']['root']);
		}
		
		// Check current action
		switch (\Input::get('act'))
		{
			case 'create':
			case 'select':
			case 'show':
				// Allow
				break;
			case 'copy':
				if (!$this->User->hasAccess('create', 'revolutionslidersp'))
				{
					$this->log('Not enough permissions to '.Input::get('act').' slider ID "'.Input::get('id').'"', 'tl_revolutionslider checkPermission', TL_ERROR);
					$this->redirect('contao/main.php?act=error');
				}
				break;
			case 'edit':
				// Dynamically add the record to the user profile
				if (!in_array(\Input::get('id'), $root))
				{
					$arrNew = $this->Session->get('new_records');
					
					if (is_array($arrNew['tl_revolutionslider']) && in_array(\Input::get('id'), $arrNew['tl_revolutionslider']))
					{
						// Add the permissions on group level
						if ($this->User->inherit != 'custom')
						{
							$objGroup = $this->Database->execute("SELECT id,revolutionsliders, revolutionslidersp FROM tl_user_group WHERE id IN(" . implode(',', array_map('intval', $this->User->groups)) . ")");
							
							while($objGroup->next())
							{
								$arrSlidersp = deserialize($objGroup->revolutionslidersp);
								if (is_array($arrSlidersp) && in_array('create', $arrSlidersp))
								{
									$arrSliders = deserialize($objGroup->revolutionsliders);
									$arrSliders[] = \Input::get('id');
									
									$this->Database->prepare("UPDATE tl_user_group SET revolutionsliders=? WHERE id=?")->execute(serialize($arrSliders), $objGroup->id);
								}
							}
						}
	
						// Add permissions on user level
						if ($this->User->inherit != 'group')
						{
							$objUser = $this->Database->prepare("SELECT id,revolutionsliders, revolutionslidersp FROM tl_user WHERE id=?")
							->limit(1)
							->execute($this->User->id);
							
							$arrSlidersp = deserialize($objUser->revolutionslidersp);
							if (is_array($arrSlidersp) && in_array('create', $arrSlidersp))
							{
								$arrSliders = deserialize($objUser->revolutionsliders);
								$arrSliders[] = \Input::get('id');

								$this->Database->prepare("UPDATE tl_user SET revolutionsliders=? WHERE id=?")->execute(serialize($arrSliders), $this->User->id);
							}
						}
	
						// Add new element to the user object
						$root[] = \Input::get('id');
						$this->User->revolutionsliders = $root;
					}
				}
			// No break;
			case 'delete':
				if (!$this->User->hasAccess('delete', 'revolutionslidersp'))
				{
					$this->log('Not enough permissions to '.Input::get('act').' slider ID "'.Input::get('id').'"', 'tl_revolutionslider checkPermission', TL_ERROR);
					$this->redirect('contao/main.php?act=error');
				}
				break;

			case 'editAll':
			case 'deleteAll':
			case 'overrideAll':
				$session = $this->Session->getData();
				if (Input::get('act') == 'deleteAll' && !$this->User->hasAccess('delete', 'revolutionslidersp'))
				{
					$session['CURRENT']['IDS'] = array();
				}
				else
				{
					$session['CURRENT']['IDS'] = array_intersect($session['CURRENT']['IDS'], $root);
				}
				$this->Session->setData($session);
				break;
	
			default:
				if (strlen(Input::get('act')))
				{
					$this->log('Not enough permissions to '.Input::get('act').' slider', 'tl_revolutionslider checkPermission', TL_ERROR);
					$this->redirect('contao/main.php?act=error');
				}
				break;
		}
	}

	/**
	 * Return the edit header button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function editHeader($row, $href, $label, $title, $icon, $attributes)
	{
		$strButton = '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.\Image::getHtml($icon, $label).'</a> ';
		if($this->User->isAdmin )
		{
			return $strButton;	
		}
		elseif($this->User->hasAccess('create', 'revolutionslidersp') && count(preg_grep('/^tl_revolutionslider::/', $this->User->alexf)) > 0 )
		{
			return $strButton;			
		}
		else
		{
			return '';
		}
	}

	/**
	 * Return the copy category button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function copyButton($row, $href, $label, $title, $icon, $attributes)
	{
		return ($this->User->isAdmin || $this->User->hasAccess('create', 'revolutionslidersp')) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.Image::getHtml($icon, $label).'</a> ' : Image::getHtml(preg_replace('/\.gif$/i', '_.gif', $icon)).' ';
	}

	/**
	 * Return the delete archive button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function deleteButton($row, $href, $label, $title, $icon, $attributes)
	{
		return ($this->User->isAdmin || $this->User->hasAccess('delete', 'revolutionslidersp')) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.Image::getHtml($icon, $label).'</a> ' : Image::getHtml(preg_replace('/\.gif$/i', '_.gif', $icon)).' ';
	}
	
	/**
	 * Return all slider templates as array
	 * @param DataContainer
	 * @return array
	 */
	public function getSliderTemplates()
	{
		return $this->getTemplateGroup('revoslider_');
	}

	/**
	 * Return all javascript templates as array
	 * @param DataContainer
	 * @return array
	 */
	public function getJsTemplates()
	{
		return $this->getTemplateGroup('js_revoslider');
	}
	
	/**
	 * Modify the palettes 
	 * @param object
	 * @return object
	 */
	public function modifyPalettes($objDC)
	{
		if( !$objDC->activeRecord )
		{
			$objDC->activeRecord = \Database::getInstance()->prepare("SELECT * FROM ".$objDC->table." WHERE id=?")->limit(1)->execute($objDC->id);
		}
		
		#if($GLOBALS['REVOLUTIONSLIDER']['isBundledVersion'])
		#{
		#	// only allow responsive slider style
		#	$GLOBALS['TL_DCA']['tl_revolutionslider']['fields']['sliderStyle']['options'] = array('responsive');
		#}
		
		if(!in_array($objDC->activeRecord->sliderStyle, array('fullscreen','fullwidth')))
		{
			unset($GLOBALS['TL_DCA'][$objDC->table]['fields']['fullScreenOffsetContainer']);
		}
	}
}
