<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 * @license		http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Core;

/**
 * Imports
 */
use RevolutionSlider\Core\Slide as Slide;

/**
 * Class file
 * RevolutionSlider SlideFactory
 * Provides methods to generate slider slides
 */
class SlideFactory
{
	/**
	 * Return a slide object by id
	 * @param integer
	 * @return object	RevoutionSliderSlide
	 */
	public static function findById($intId)
	{
		$objSlide = \RevolutionSlider\Models\Slides::findByPk($intId);
		
		if($objSlide === null || !$objSlide->published)
		{
			return null;
		}
		
		$objSlide = new Slide($objSlide->row());
		
		return $objSlide;
	}
	
	/**
	 * Return all RevolutionSliderSlide objects from a slider
	 * @param integer
	 * @return array
	 */
	public static function findAllBySlider($intSlider)
	{
		// fetch slides
		$objDatabase = \Database::getInstance();
		$objDbSlides = $objDatabase->prepare("SELECT * FROM tl_revolutionslider_slides WHERE pid=? AND published=1 ORDER BY sorting")->execute($intSlider);
		
		if($objDbSlides->numRows < 1) {	return null; }
		
		$arrReturn = array();
		while($objDbSlides->next())
		{
			$objSlide = new Slide($objDbSlides->row());
			$arrReturn[] = $objSlide;
		}
		
		return $arrReturn;
	}
	
	/**
	 * Return a slide object by id
	 * @param integer
	 * @return array
	 */
	public static function findByIds($arrIds)
	{
		// fetch slides
		$objDatabase = \Database::getInstance();
		$objDbSlides = $objDatabase->prepare("SELECT * FROM tl_revolutionslider_slides WHERE id IN (".implode(',', $arrIds).") AND published=1")->execute();
		
		if($objDbSlides->numRows < 1) {	return null; }
		
		$arrReturn = array();
		while($objDbSlides->next())
		{
			$objSlide = new Slide($objDbSlides->row());
			$arrReturn[] = $objSlide;
		}
		
		return $arrReturn;
	}
}